@@ -35,6 +35,7 @@ def _priority(x):
 # standardize the condition string
 def _standardize_seq(seq):
     # delete spaces in the string
+    seq = seq.strip(" ")
     seq = re.sub('\s*!\s*','!',seq)
     seq = re.sub('\s*\(\s*','(',seq)
     seq = re.sub('\s*\)\s*',')',seq)
@@ -46,8 +47,6 @@ def _standardize_seq(seq):
     seq = re.sub('\s*<\s*','<',seq)
     seq = re.sub('\s*>=\s*','>=',seq)
     seq = re.sub('\s*<=\s*','<=',seq)
-    while(seq[-1]==' '):
-        seq = seq[:-1]
     return seq
 
 
@@ -495,8 +494,6 @@ def build_index():
         no return but print a warning when the index has been built
     """
 
-    ## [0601] 是不是需要改这边
-
     # check if the index exists in the table
     index_list = client_global.list_search_index(_tablename)
     if index_list and (_tablename, 'metadata')  in index_list:
@@ -525,7 +522,7 @@ def build_index():
         client_global.create_search_index(_tablename, "metadata", index_meta)
 
 
-def query_cells(metadata_conditions="",include_children=False,exlude_Unclassified=False):
+def query_cells(metadata_conditions="",include_children=False,exlude_Unclassified=False,print_message=True):
     """query cells and return the list of their primary keys
     
         This function is used to query cells satisfing the metadata conditions in the OTS table on the server. The conditions on the metadata should be a structured string which is a combination of several logical expressions. 
@@ -604,7 +601,11 @@ def query_cells(metadata_conditions="",include_children=False,exlude_Unclassifie
     
     # build the list of the primary keys of the quried cells
     rows_to_get = [x[0] for x in results]
-    print("%d cells found" %len(results))    
+
+    # [1108] print the number of cells
+    if print_message:
+        print("%d cells found" %len(results))
+
     return rows_to_get
 
 
@@ -637,6 +638,8 @@ def _para_get_column(arg_bag):
     ### [0601] avoid errors from NA in the row
     if not return_row is None:
         return [(x[0],x[1]) for x in return_row.primary_key] + [(x[0],x[1]) for x in return_row.attribute_columns]
+    else:
+        return None
     #   return ([x[1] for x in return_row.primary_key] + [x[1] for x in return_row.attribute_columns])
 
 
@@ -682,9 +685,9 @@ def get_columnsbycell_para(rows_to_get=None,cols_to_get=None,col_filter=None,do_
     if rows_to_get == None:
         rows_to_get = query_cells(exlude_Unclassified=exlude_Unclassified)
     
-    ## [0601] transform the hcad_name in the colnames into uHAF_name
-    #if "hcad_name" in colnames:
-    #    colnames[colnames.index("hcad_name")] = "uHAF_name"
+    # [1108] replace the uHAF_name to hcad_name in the cols_to_get
+    if not cols_to_get == None:
+        cols_to_get = ["hcad_name" if i == "uHAF_name" else i for i in cols_to_get]
     
     arg_bag = [(rows_to_get[i],cols_to_get,col_filter) for i in range(len(rows_to_get))]
     try:
@@ -697,7 +700,7 @@ def get_columnsbycell_para(rows_to_get=None,cols_to_get=None,col_filter=None,do_
             df_get = pool.map(_para_get_column,arg_bag)
             pool.close()
             pool.join()
-            
+
     while None in df_get:
         df_get.remove(None)
         
@@ -707,6 +710,10 @@ def get_columnsbycell_para(rows_to_get=None,cols_to_get=None,col_filter=None,do_
         if do_transfer:
             df_get = pd.DataFrame.from_dict([dict(df_get[i]) for i in range(len(df_get))]) 
             df_get = df_get.set_index(['cid'])
+
+            # [1108] replace the hcad_name in the colnames with uHAF_name
+            df_get.columns = ["uHAF_name" if i == "hcad_name" else i for i in df_get.columns ]
+
     return df_get
 
 
@@ -753,7 +760,10 @@ def get_columnsbycell(rows_to_get=None,cols_to_get=None,col_filter=None,do_trans
     if rows_to_get == None:
         rows_to_get = query_cells(exlude_Unclassified)
         
-        
+    # [1108] replace the uHAF_name to hcad_name in the cols_to_get
+    if not cols_to_get == None:
+        cols_to_get = ["hcad_name" if i == "uHAF_name" else i for i in cols_to_get]
+
     request = tablestore.BatchGetRowRequest()
     df_get = []
     colnames = None
@@ -780,6 +790,10 @@ def get_columnsbycell(rows_to_get=None,cols_to_get=None,col_filter=None,do_trans
         if do_transfer:
             df_get = pd.DataFrame.from_dict([dict(df_get[i]) for i in range(len(df_get))])
             df_get.set_index(['cid'])
+
+            # [1108] replace the hcad_name in the colnames with uHAF_name
+            df_get.columns = ["uHAF_name" if i == "hcad_name" else i for i in df_get.columns ]
+
         return df_get
 
 
@@ -810,6 +824,11 @@ def get_column_set(rows_to_get=None, col_to_get=None, col_filter=None,exlude_Unc
         A set of all unique values in the selected column
     """
     if(len(col_to_get) == 1):
+
+        # [1108] replace uHAF_name with hcad_name
+        if col_to_get == ['uHAF_name']:
+            col_to_get = ['hcad_name']
+
         if rows_to_get == None:
             rows_to_get = query_cells(exlude_Unclassified)
         
@@ -849,6 +868,10 @@ def get_all_rows(cols_to_get = []):
     if cols_to_get:
         col_return_type = tablestore.ColumnReturnType.SPECIFIED
 
+    # [1108] replace the uHAF_name to hcad_name in the cols_to_get
+    if not cols_to_get == None:
+        cols_to_get = ["hcad_name" if i == "uHAF_name" else i for i in cols_to_get]
+
     while not all_rows or next_token:
         # [0620] update for tablestore 5.2.1
         search_result = client_global.search(_tablename, 'metadata',
@@ -968,6 +991,8 @@ def update_batch(rows_to_update, update_sets, thread_num = 5):
         
         If any operation fails, this function will return the list of all update status. In i-th element, 0 means success and -1 means failure in the i-th operation.
     """
+    
+    
     arg_bag = [ (rows_to_update[i],update_sets[i]) for i in range(len(rows_to_update))]
     ## [0601] add try... except to avoid error
     try:
@@ -985,6 +1010,105 @@ def update_batch(rows_to_update, update_sets, thread_num = 5):
         return b
     else:
         return 0
+    
+    
+    
+# [1108] add new interfaces with new names
+
+def search_metadata(metadata_conditions="",include_children=False,exlude_Unclassified=False,print_message=True):
+    """query cells and return the list of their primary keys
+    
+        This function is used to query cells satisfing the metadata conditions in the OTS table on the server. The conditions on the metadata should be a structured string which is a combination of several logical expressions. 
+        
+        Users should remember that this function only enable them to query cell based on the conditions on the metadata columns in the index while conditions on gene expression should be set in the get_columnsbycell function or get_columnsbycell_para function.
+    
+    
+    Parameters
+    ----------
+    metadata_conditions: str
+        It is a structured string which is a combination of several logical expressions.
+
+        Each logical expression should be in the following forms:
+            field_name1 == value1,                          here '==' means equal
+
+            field_name2 <> value2,                          here '<>' means unequal
+
+        Three symbols are used for logical operation between expressions:
+            logical_expression1 && logical_expression2,     here '&&' means AND operation
+
+            logical_expression1 || logical_expression2,     here '||' means OR operation
+
+            ！ logical_expression1,                         here '!' means not NOT operation
+
+        Brackets are allowed and the priorities of the logical operations are as common. The metadata condition string is also robust to the space character.
+
+        Here are some examples of legal condition strings:
+            '(user_id == 2 && organ == Heart) || (user_id == 3 && organ <> Brain)'
+
+            'organ == Lung && !seq_tech == 10X  '
+
+            'organ == Heart &&(cell_type == Fibrocyte || region <> atria)'
+        
+        The default value of this parameter is '', which will get all rows in the database.
+
+    include_children: bool 
+        include_children is the parameter deciding if subtypes should be included in the query with condition on cell type. For example, when we query "cell_type == T cell" and set include_children as True, 
+        cells in the query result would contain T cell and the subtypes of T cells like CD4 T cell, CD8 T cell and so on; otherwise, only T cells would exist in the query result.
+    
+    Returns
+    -------
+    list
+        The return is a list of the primary keys of quried cells. Each element is a list containing a tuple with the name of primary key and its value: [('cid',2021119)].
+        
+        Here is an example return with 2000 quried cells:
+
+            [[('cid',2000001)],[('cid',2000002)],[('cid',2000003)],[('cid',2000004)],...,[('cid',2002000)]]
+
+        The primary key list can be used for downstream data downloading or table updating.
+    """
+    return query_cells(metadata_conditions,include_children,exlude_Unclassified,print_message)
+
+
+def set_gene_condition(gene_condition):
+    """transform a gene-condition string into a OTS column condition
+
+    This function will analyze the logical expression string of the gene condition for cell searching and generate a combined column condition of OTS database.
+
+    Parameters
+    ----------
+    gene_condition: str
+        It is a structured string which is a combination of several logical expressions.
+
+        Each logical expression should be in the following forms:
+            gene_name1 == value1,                          here '==' means equal
+
+            gene_name2 <> value2,                          here '<>' means unequal
+            
+            gene_name3 > value3,                          here '>' means larger than 
+            
+            gene_name4 < value4,                          here '<' means smaller than
+            
+            gene_name5 >= value5,                          here '>=' means not smaller than
+            
+            gene_name6 <= value6,                          here '<=' means not larger than
+
+        Three symbols are used for logical operation between expressions:
+            logical_expression1 && logical_expression2,     here '&&' means AND operation
+
+            logical_expression1 || logical_expression2,     here '||' means OR operation
+
+            ！ logical_expression1,                         here '!' means not NOT operation
+
+        Brackets are allowed and the priorities of the logical operations are as common. The metadata condition string is also robust to the space character.
+
+        Here are some examples of legal condition strings:
+            '(CD3D > 2 && CD3E >= 0.1) || (PTPRC <= 3 && CD8A >= 0.01)'
 
 
 
+    Returns
+    -------
+    tablestore.CompositeColumnCondition
+        the column condition for tablestore to seach cells
+    """
+    return seq2filter(gene_condition)
\ No newline at end of file
